Package: BRRAT
Title: Bayesian Regression Robustness Assessment Test
Version: 0.0.2
Authors@R: 
    person("Matt", "Gibbs", email = "matt.gibbs@csiro.au",
    role = c("aut", "cre"))
URL: https://github.com/csiro/hydro_BRRAT_Package
BugReports: https://github.com/csiro/hydro_BRRAT_Package/issues
Description: Tests for a linear relationship in the log ratio between an observed and simulated series and an independent variable. Typically this the error in modelled streamflow at an annual time scale, and a rainfall input. The approach allows for multiple sites as random factors and for multiple replicates of the simulated values. The approach is outlined in Gibbs et al. (2026) in review.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Biarch: true
Depends: R (>= 4.1.0)
Imports: methods, Rcpp (>= 1.1.0), RcppParallel (>= 5.0.1), rstan (>=
        2.18.1), rstantools (>= 2.5.0), MASS, ggplot2, dplyr
Suggests: covr, testthat (>= 3.0.0)
Config/testthat/edition: 3
LinkingTo: BH (>= 1.66.0), Rcpp (>= 1.1.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
SystemRequirements: GNU make
NeedsCompilation: yes
Packaged: 2026-04-23 04:45:59 UTC; gib417
Author: Matt Gibbs [aut, cre]
Maintainer: Matt Gibbs <matt.gibbs@csiro.au>
Repository: CRAN
Date/Publication: 2026-04-24 20:30:11 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-24 23:28:02 UTC; unix
Archs: BRRAT.so.dSYM

Package: BRRAT
Title: Bayesian Regression Robustness Assessment Test
Version: 0.0.2
Authors@R: 
    person("Matt", "Gibbs", email = "matt.gibbs@csiro.au",
    role = c("aut", "cre"))
URL: https://github.com/csiro/hydro_BRRAT_Package
BugReports: https://github.com/csiro/hydro_BRRAT_Package/issues
Description: Tests for a linear relationship in the log ratio between an observed and simulated series and an independent variable. Typically this the error in modelled streamflow at an annual time scale, and a rainfall input. The approach allows for multiple sites as random factors and for multiple replicates of the simulated values. The approach is outlined in Gibbs et al. (2026) in review.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Biarch: true
Depends: R (>= 4.1.0)
Imports: methods, Rcpp (>= 1.1.0), RcppParallel (>= 5.0.1), rstan (>=
        2.18.1), rstantools (>= 2.5.0), MASS, ggplot2, dplyr
Suggests: covr, testthat (>= 3.0.0)
Config/testthat/edition: 3
LinkingTo: BH (>= 1.66.0), Rcpp (>= 1.1.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
SystemRequirements: GNU make
NeedsCompilation: yes
Packaged: 2026-04-23 04:45:59 UTC; gib417
Author: Matt Gibbs [aut, cre]
Maintainer: Matt Gibbs <matt.gibbs@csiro.au>
Repository: CRAN
Date/Publication: 2026-04-24 20:30:11 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 16:50:09 UTC; unix
Archs: BRRAT.so.dSYM

# Generated by roxygen2: do not edit by hand

export(BRRAT)
export(export_BRRAT_csv)
export(plot_BRRAT_by_id)
export(plot_BRRAT_population)
export(summarise_BRRAT)
import(MASS)
import(Rcpp)
import(ggplot2)
import(methods)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(ggplot2,aes)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_minimal)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
importFrom(stats,lm)
importFrom(stats,qchisq)
useDynLib(BRRAT, .registration = TRUE)

# Bayesian Regression Robustness Assessment Test (BRRAT)

<!-- badges: start -->

[![R-CMD-check](https://github.com/csiro/hydro_BRRAT_Package/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/csiro/hydro_BRRAT_Package/actions/workflows/R-CMD-check.yaml)[![DOI](https://img.shields.io/badge/DOI-10.5281%2Fzenodo.17957953-blue)](https://doi.org/10.5281/zenodo.17957953)

<!-- badges: end -->

## Overview

The Robustness Assessment Test (RAT) developed by [Nicolle et al. (2021)](https://hess.copernicus.org/articles/25/5013/2021/) tests for a relationship between the relative bias in annual streamflow and climatic variables, 
e.g. rainfall, temperature or humidity. If a statistically significant relationship is identified it is suggested that the rainfall runoff model that generated the annual streamflow series cannot be safely used for climate change impact studies. 

This package implements an extension that compares the slope of the linear regression line between relative model error and a rainfall index, 
with a slope closer to zero representative of a more robust model. This extension allows for a improvements in model structure or calibration method to be tested more easily compared to the original RAT pass/fail test. 
The regression parameters are fit using Bayesian regression (making use of the rstan package), with the modified test referred to as the Bayesian Regression Robustness Assessment Test (BRRAT). 

A hierarchical linear regression approach is used if multiple catchments are considered simultaneously.

## Getting Started

Package has been submitted to CRAN. Once on CRAN, install with install.packages("BRRAT")

To install from github source, the easiest approach is: devtools::install_github("csiro/hydro_BRRAT_Package")

## Usage 

```r
# Generate 2 models of annual streamflow
samples <- 50

# Generate some annual rainfall totals
Clim <- sort(pmax(0, rnorm(samples, 600, 100)))

# Calculate annual runoff from tanh curve
observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150)/450))
Obs <- data.frame(P = Clim, Q = observed, year = 1:length(Clim))

# Non-robust model, underestimates the wetter years
Sim1 <- observed * seq(1, 0.7, length.out = length(observed)) * rnorm(samples, 1, 0.05)
Sim1 <- data.frame(Q = Sim1, year = 1:length(Clim))

# Robust model, only random error
Sim2 <- observed * rnorm(samples, 1, 0.05)
Sim2 <- data.frame(Q = Sim2, year = 1:length(Clim))

# Apply BRRAT test to both (chains and iter reduced to speed up example)
fit1 <- BRRAT(Sim1, Obs, chains = 1, iter = 500)
fit2 <- BRRAT(Sim2, Obs, chains = 1, iter = 500)

# Compare posterior distributions of slope — fit2 centred on 0, fit1 has negative slope
dat <- data.frame(
  b  = c(fit1$beta_draws, fit2$beta_draws),
  id = c(rep("fit1", length(fit1$beta_draws)), rep("fit2", length(fit2$beta_draws)))
)
ggplot(dat) + geom_density(aes(b, fill = id))

# Or use the built-in plotting function
plot_BRRAT_population(fit1)
plot_BRRAT_population(fit2)

# Summarise posterior statistics
summarise_BRRAT(fit1)
summarise_BRRAT(fit2)
```

## Citation

Gibbs MS, Wang B, Crosbie R, Montazeri M, Vaze J and Yang A (2026) Improving distributed hydrological model robustness through multi-variable calibration. Hydrological Processes, In Press.

## Contribute

Feel free to add pull requests, raise issues here on github or email.

#' Plot population-level regression Qe ~ P with uncertainty band
#'
#' @title Population-level line and credible band
#' @description
#' Plots the **population-level** regression line \eqn{\alpha + \beta x} and its
#' equal-tail credible band by transforming the x-grid to the centered/scaled space
#' used during fitting. The plot overlays a (potentially downsampled) scatter of the
#' original observations for context.
#'
#' @param fit_obj A fitted object returned by [BRRAT()], of class `BRRAT`.
#' @param n_grid Integer; number of x points for the prediction grid (default `200`).
#' @param level Credible interval level for the band (default `0.95`).
#'
#' @return A `ggplot2` object showing the population regression line and credible band.
#'
#' @details
#' The function extracts posterior draws of `alpha` and `beta` from the Stan fit and
#' evaluates \eqn{\mu(x) = \alpha + \beta x} along a grid of x values on the original
#' scale, internally converting to the centered/scaled space via
#' \eqn{x_{\mathrm{std}} = (x - x_{\mathrm{center}}) / x_{\mathrm{scale}}}.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' samples <- 30
#' Clim <- sort(pmax(0, rnorm(samples, 600, 100)))
#' observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150) / 450))
#' Obs <- data.frame(P = Clim, Q = observed, year = 1:samples)
#' Sim <- data.frame(Q = observed * rnorm(samples, 1, 0.05), year = 1:samples)
#' res <- BRRAT(Sim, Obs, chains = 1, iter = 500)
#' p_pop <- plot_BRRAT_population(res, n_grid = 200, level = 0.95)
#' print(p_pop)
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_point geom_ribbon geom_line labs theme_minimal
#' @export
plot_BRRAT_population <- function(fit_obj, n_grid = 200, level = 0.95) {
  stopifnot(inherits(fit_obj, 'BRRAT'))

  draws <- rstan::extract(fit_obj$fit, pars = c('alpha','beta'))
  alpha <- draws$alpha
  beta  <- draws$beta

  x_min  <- min(fit_obj$data$x, na.rm = TRUE)
  x_max  <- max(fit_obj$data$x, na.rm = TRUE)
  x_std <- seq(x_min, x_max, length.out = n_grid)

  mu_draws <- sapply(x_std, function(xi) alpha + beta * xi)
  mu_mean  <- apply(mu_draws, 2, mean)
  q_lo     <- (1 - level) / 2
  q_hi     <- 1 - q_lo
  mu_lo    <- apply(mu_draws, 2, stats::quantile, probs = q_lo)
  mu_hi    <- apply(mu_draws, 2, stats::quantile, probs = q_hi)

  df_plot <- data.frame(P = x_std, mu_mean = mu_mean, mu_lo = mu_lo, mu_hi = mu_hi)

  data_sc <- data.frame(x=fit_obj$data$x,y=fit_obj$data$y)
  if (nrow(data_sc) > 20000) {
    data_sc <- data_sc[sample(seq_len(nrow(data_sc)), 20000), ]
  }

  if (!requireNamespace('ggplot2', quietly = TRUE)) stop('ggplot2 is required.')

  ggplot2::ggplot() +
    ggplot2::geom_point(data = data_sc,
                        ggplot2::aes(x = .data$x, y = .data$y), alpha = 0.15, size = 0.6) +
    ggplot2::geom_ribbon(data = df_plot,
                         ggplot2::aes(x = .data$P, ymin = .data$mu_lo, ymax = .data$mu_hi),
                         fill = '#1f77b4', alpha = 0.15) +
    ggplot2::geom_line(data = df_plot, ggplot2::aes(x = .data$P, y = .data$mu_mean),
                       colour = '#1f77b4', linewidth = 1) +
    ggplot2::labs(x = 'P (normalized rainfall)', y = 'Qerr (transformed)',
                  title = sprintf('Population regression: Qerr ~ P (%.0f%% credible band)', level * 100)) +
    ggplot2::theme_minimal()
}


#' Plot per-site lines (random intercepts & slopes) with credible intervals
#'
#' @title Per-site partial-pooling lines and credible bands
#' @description
#' Plots **site-specific** regression lines using draws of `intercept_id[j]` and
#' `slope_id[j]` for each site `ID`. Requires more than 1 ID in the initial
#' regression by [BRRAT()]. The output overlays lines and credible bands
#' across sites; optionally facetted into separate panels for readability.
#'
#' @param fit_obj A fitted object returned by [BRRAT()], of class `BRRAT`.
#' @param n_grid Integer; number of x points per site for the prediction grids
#' (default `150`).
#' @param level Credible interval level (default `0.80`) used for site-specific bands.
#' @param facet Logical; if `TRUE`, facet the plot by `ID`; otherwise overlay all sites
#' with colour and fill aesthetics (default `TRUE`).
#'
#' @return A `ggplot2` object showing per-site lines and credible intervals.
#'
#' @details
#' For site \eqn{j}, the line is computed as:
#' \deqn{\mu_j(x) = \mathrm{intercept\_id}_j + \mathrm{slope\_id}_j \, x_{\mathrm{std}}}{
#' mu_j(x) = intercept_id[j] + slope_id[j] * x_std}
#' evaluated on a grid of x values transformed to the centered/scaled space used
#' during fitting. The function reconstructs equal-tail credible bands by propagating
#' posterior draws of `intercept_id[j]` and `slope_id[j]`.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' samples <- 30
#' make_site <- function(id) {
#'   Clim <- sort(pmax(0, rnorm(samples, 600, 100)))
#'   observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150) / 450))
#'   list(
#'     Obs = data.frame(P = Clim, Q = observed, year = 1:samples, ID = id),
#'     Sim = data.frame(Q = observed * rnorm(samples, 1, 0.05), year = 1:samples, ID = id)
#'   )
#' }
#' s1 <- make_site("A")
#' s2 <- make_site("B")
#' Obs <- rbind(s1$Obs, s2$Obs)
#' Sim <- rbind(s1$Sim, s2$Sim)
#' res <- BRRAT(Sim, Obs, chains = 1, iter = 500)
#' p_sites <- plot_BRRAT_by_id(res, n_grid = 150, level = 0.80, facet = TRUE)
#' print(p_sites)
#' }
#'
#' @importFrom ggplot2 ggplot aes geom_point geom_ribbon geom_line labs theme_minimal facet_wrap guides
#' @export
plot_BRRAT_by_id <- function(fit_obj, n_grid = 150, level = 0.80, facet = TRUE) {
  stopifnot(inherits(fit_obj, 'BRRAT'))

  id_levels <- fit_obj$id_levels
  J <- length(id_levels)

  # Extract per-ID generated quantities (iterations x J)
  intercept_draws <- rstan::extract(fit_obj$fit, pars = 'intercept_id')[[1]]
  slope_draws     <- rstan::extract(fit_obj$fit, pars = 'slope_id')[[1]]

  x_min  <- min(fit_obj$data$x, na.rm = TRUE)
  x_max  <- max(fit_obj$data$x, na.rm = TRUE)
  x_std <- seq(x_min, x_max, length.out = n_grid)

  q_lo <- (1 - level) / 2
  q_hi <- 1 - q_lo

  out_list <- vector('list', J)
  for (j in seq_len(J)) {
    mu_draws_j <- sapply(x_std, function(xi) intercept_draws[, j] + slope_draws[, j] * xi)
    mu_mean <- apply(mu_draws_j, 2, mean)
    mu_lo   <- apply(mu_draws_j, 2, stats::quantile, probs = q_lo)
    mu_hi   <- apply(mu_draws_j, 2, stats::quantile, probs = q_hi)
    out_list[[j]] <- data.frame(ID = id_levels[j], P = x_std,
                                mu_mean = mu_mean, mu_lo = mu_lo, mu_hi = mu_hi)
  }
  df_lines <- do.call(rbind, out_list)

  # Downsample points per ID if very large
  data_sc <- data.frame(x=fit_obj$data$x,
                        y=fit_obj$data$y,
                        ID = fit_obj$id_levels[fit_obj$data$group])
  if (nrow(data_sc) > 30000) {
    data_sc <- do.call(rbind, lapply(split(data_sc, data_sc$ID), function(dd) {
      keep <- min(5000, nrow(dd))
      dd[sample(seq_len(nrow(dd)), keep), ]
    }))
  }

  if (!requireNamespace('ggplot2', quietly = TRUE)) stop('ggplot2 is required.')

  p <- ggplot2::ggplot() +
    ggplot2::geom_point(data = data_sc, ggplot2::aes(x = .data$x, y = .data$y, colour = .data$ID),
                        alpha = 0.12, size = 0.6) +
    ggplot2::geom_ribbon(data = df_lines, ggplot2::aes(x = .data$P, ymin = .data$mu_lo, ymax = .data$mu_hi, fill = .data$ID),
                         alpha = 0.15) +
    ggplot2::geom_line(data = df_lines, ggplot2::aes(x = .data$P, y = .data$mu_mean, colour = .data$ID),
                       linewidth = 1) +
    ggplot2::labs(x = 'P (normalized)', y = 'Qerr (transformed)',
                  title = sprintf('Per-site partial-pooling lines (%.0f%% credible bands)', level * 100)) +
    ggplot2::theme_minimal()

  if (facet) {
    p <- p + ggplot2::facet_wrap(~ID, scales = 'free_y', ncol = 3) +
      ggplot2::guides(colour = 'none', fill = 'none')
  }
  return(p)
}

test_that("Population plot returns a ggplot object", {
  set.seed(30)
  samples <- 50
  #' #generate some annual rainfall totals
  Clim <- sort(pmax(0,rnorm(samples,600,100)))
  #calculate annual runoff from tanh curve
  observed <- pmax(0,(Clim - 150) - 450 * tanh((Clim - 150)/450))
  Obs <- data.frame(P=Clim,Q=observed,year=1:length(Clim),ID=1)
  #'
  #' #non-robust model, underestimate the wetter years
  Sim <- observed*seq(1,0.7,length.out=length(observed))*rnorm(samples,1,0.05)
  Sim <- data.frame(Q=Sim,year=1:length(Clim),ID=1)

  expect_warning(
  fit <- BRRAT(Sim, Obs, iter = iters, warmup = warmup, chains = chains),
  regexp = "Samples Size")

  p   <- plot_BRRAT_population(fit, n_grid = 50, level = 0.8)
  expect_s3_class(p, "ggplot")
})

test_that("By-ID plot and statistics are produced when model has per-site parameters", {

  set.seed(30)
  samples <- 50
  #' #generate some annual rainfall totals
  Clim <- sort(pmax(0,rnorm(samples,600,100)))
  #calculate annual runoff from tanh curve
  observed <- pmax(0,(Clim - 150) - 450 * tanh((Clim - 150)/450))
  Obs <- data.frame(P=c(Clim,Clim),
                    Q=c(observed,observed*0.8),
                    year=c(1:length(Clim),1:length(Clim)),
                    ID=c(rep(1,length(Clim)),rep(2,length(Clim))))

  #' #non-robust model, underestimate the wetter years
  Sim <- Obs[,c("year","Q","ID")]
  Sim$Q <- Sim$Q*rnorm(samples*2,1,0.05) #add some noise, but no trend

  expect_warning(
    fit <- BRRAT(Sim, Obs, iter = iters, warmup = warmup, chains = chains),
    regexp = "Samples Size")

  p   <- plot_BRRAT_population(fit, n_grid = 50, level = 0.8)
  expect_s3_class(p, "ggplot")

  p <- plot_BRRAT_by_id(fit, n_grid = 50, level = 0.8, facet = TRUE)
  expect_s3_class(p, "ggplot")

  out <- summarise_BRRAT(fit, level = 0.95)

  expect_type(out, "list")
  expect_true(all(c("global", "per_id") %in% names(out)))

  # Global table should have slope estimates (median and interval)
  expect_true(all(c("param", "mean", "sd",  "q_lo", "q_hi", "prob_gt0", "prob_lt0","p_nonzero") %in% names(out$global)),
              info = "Global summary must include slope and probability statistics")

  # test per_id outputs exist
  expect_true(all(c("ID", "intercept_mean", "intercept_sd", "intercept_q_lo",
                      "intercept_q_hi", "slope_mean", "slope_sd", "slope_q_lo",
                      "slope_q_hi" ) %in% names(out$per_id)),info = "per_id summary should include intercept and slope information")
  expect_true(nrow(out$per_id)==2, info = "there should be two rows in out$per_id for the two IDs tested")

})

#' Summarise global slope and per-site coefficients into tidy tables
#'
#' @title Summarise population and site-level posterior summaries
#' @description
#' Computes tidy posterior summaries for the **population slope** (`beta`) and
#' **per-site intercepts and slopes** (from `intercept_id` and `slope_id` generated
#' quantities) returned by the Stan fit. The function reports posterior means,
#' standard deviations, equal-tail credible intervals at a chosen level, and a
#' two-sided tail probability for non-zero slope at the population level.
#'
#' @details
#' The two-sided tail probability is computed as:
#' \deqn{p_{\mathrm{nonzero}} = 2\min\{ \Pr(\beta > 0 \mid \mathrm{data}), \Pr(\beta < 0 \mid \mathrm{data}) \}}{
#' p_nonzero = 2 * min(Pr(beta > 0 | data), Pr(beta < 0 | data))}
#' which is analogous to a two-sided p-value in spirit, but is a **Bayesian posterior tail
#' probability**. It shrinks toward 0 as the posterior mass concentrates away from zero.
#'
#' @param fit_obj A fitted object returned by [BRRAT()], of class `BRRAT`.
#' @param level Credible interval level (default `0.95`). The lower and upper
#' intervals are computed as `(1 - level) / 2` and `1 - (1 - level) / 2`.
#'
#' @return A list with one or two data frames:
#' \itemize{
#'   \item `global`: one row summarising the population slope `beta` (mean, sd,
#'   lower/upper credible interval, `prob_gt0`, `prob_lt0`, and `p_nonzero`).
#'   \item `per_id`: if applicable (more than one site `ID` in the data) one
#'   row per site `ID` with posterior summaries for
#'   `intercept_id` and `slope_id` (mean, sd, lower/upper credible intervals).
#' }
#'
#' @section Notes:
#' Assumes the Stan model emits `intercept_id` and `slope_id` in `generated quantities`,
#' i.e., per-site fully-composed coefficients:
#' \deqn{\mathrm{intercept\_id}_j = \alpha + b_{0j},\quad \mathrm{slope\_id}_j = \beta + b_{1j}}{
#' intercept_id[j] = alpha + b0[j]; slope_id[j] = beta + b1[j]}
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' samples <- 30
#' Clim <- sort(pmax(0, rnorm(samples, 600, 100)))
#' observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150) / 450))
#' Obs <- data.frame(P = Clim, Q = observed, year = 1:samples)
#' Sim <- data.frame(Q = observed * rnorm(samples, 1, 0.05), year = 1:samples)
#' res <- BRRAT(Sim, Obs, chains = 1, iter = 500)
#' tabs <- summarise_BRRAT(res, level = 0.95)
#' tabs$global
#' head(tabs$per_id)
#' }
#'
#' @export
summarise_BRRAT <- function(fit_obj, level = 0.95) {
  stopifnot(inherits(fit_obj, 'BRRAT'))
  sf   <- fit_obj$fit
  q_lo <- (1 - level) / 2
  q_hi <- 1 - q_lo

  # Population slope beta
  beta_draws <- rstan::extract(sf, pars = 'beta')[[1]]
  global <- data.frame(
    param     = 'beta',
    mean      = mean(beta_draws),
    sd        = stats::sd(beta_draws),
    q_lo      = stats::quantile(beta_draws, probs = q_lo),
    q_hi      = stats::quantile(beta_draws, probs = q_hi),
    prob_gt0  = mean(beta_draws > 0),
    prob_lt0  = mean(beta_draws < 0),
    p_nonzero = 2 * min(mean(beta_draws > 0), mean(beta_draws < 0))
  )

  if(fit_obj$data$J>1){
    # Per-ID intercepts and slopes
    intercept_draws <- rstan::extract(sf, pars = 'intercept_id')[[1]]  # D x J
    slope_draws     <- rstan::extract(sf, pars = 'slope_id')[[1]]      # D x J
    id_levels       <- fit_obj$id_levels

    per_id <- data.frame(
      ID               = id_levels,
      intercept_mean   = apply(intercept_draws, 2, mean),
      intercept_sd     = apply(intercept_draws, 2, stats::sd),
      intercept_q_lo   = apply(intercept_draws, 2, stats::quantile, probs = q_lo),
      intercept_q_hi   = apply(intercept_draws, 2, stats::quantile, probs = q_hi),
      slope_mean       = apply(slope_draws, 2, mean),
      slope_sd         = apply(slope_draws, 2, stats::sd),
      slope_q_lo       = apply(slope_draws, 2, stats::quantile, probs = q_lo),
      slope_q_hi       = apply(slope_draws, 2, stats::quantile, probs = q_hi)
    )
  }else{
    per_id <- NULL
  }

  list(global = global, per_id = per_id)
}

#' Export summary tables to CSV files
#'
#' @title Export global and per-site summaries to CSV
#' @description
#' Writes the output of [summarise_BRRAT()] to two CSV files: one for the
#' population slope (`file_global`) and one for per-site coefficients (`file_per_id`).
#'
#' @param fit_obj A fitted object returned by [BRRAT()], of class `BRRAT`.
#' @param file_global Path/filename for the global table CSV. No default is
#'   provided; the user must supply an explicit path (e.g. via [tempfile()] or
#'   a user-chosen location).
#' @param file_per_id If applicable to the model, path/filename for the
#'   per-site table CSV. No default is provided; the user must supply an
#'   explicit path.
#' @param level Credible interval level passed to [summarise_BRRAT()] (default `0.95`).
#'
#' @return (Invisibly) a list with the two data frames written:
#' `list(global = <data.frame>, per_id = <data.frame>)`.
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' samples <- 30
#' Clim <- sort(pmax(0, rnorm(samples, 600, 100)))
#' observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150) / 450))
#' Obs <- data.frame(P = Clim, Q = observed, year = 1:samples)
#' Sim <- data.frame(Q = observed * rnorm(samples, 1, 0.05), year = 1:samples)
#' res <- BRRAT(Sim, Obs, chains = 1, iter = 500)
#' export_BRRAT_csv(res,
#'   file_global = tempfile(fileext = ".csv"),
#'   file_per_id = tempfile(fileext = ".csv"),
#'   level = 0.95
#' )
#' }
#'
#' @export
export_BRRAT_csv <- function(fit_obj,
                                file_global,
                                file_per_id,
                                level = 0.95) {
  tabs <- summarise_BRRAT(fit_obj, level = level)
  utils::write.csv(tabs$global,  file_global,  row.names = FALSE)
  if(!is.null(tabs$per_id)){
    utils::write.csv(tabs$per_id,  file_per_id,  row.names = FALSE)
  }
  invisible(tabs)
}

#' The 'BRRAT' package.
#'
#' @description Function to apply the Bayesian Regression Robustness Assessment Test,
#' to evaluate if model simulations are independent of the climate input used,
#' suggesting a robust model.
#'
#' This package makes use of Stan to fit the linear regression model,
#' and the precompiled model created for the package with rstantools.
#'
#' @docType package
#' @name BRRAT-package
#' @aliases BRRAT-package
#' @useDynLib BRRAT, .registration = TRUE
#' @import methods
#' @import Rcpp
#' @import MASS
#' @import ggplot2
#' @importFrom rstan sampling
#' @importFrom rstantools rstan_config
#' @importFrom RcppParallel RcppParallelLibs
#' @importFrom stats lm qchisq
#'
#' @references
#' Stan Development Team (NA). RStan: the R interface to Stan. R package version 2.32.6. https://mc-stan.org
#'
#' Gibbs et al. (2026) in review
#'
"_PACKAGE"
NULL

#' Bayesian Regression Robustness Assessment Test
#'
#' Function to apply the BRRAT. A linear regression is fit between the error between the transformed `Sim`
#' and `Obs` data and the corresponding `Clim` data, to test for a dependent
#' relationship between the error and climate, indicating a model that is not
#' robust to the input climate data. Stan is used to fit the linear regression,
#' and hence a distribution of slope values are returned.
#'
#' Multiple catchments are supported through random factors in the hierarchical
#' linear regression model,indicated by different values in an `ID` column.
#'
#' Multiple replicates of the Simulated time series are supported, matched based
#' on the `year` column.
#'
#' @param Sim Data frame type containing time series of simulated values (typically annual streamflow).
#' @param Obs Data frame type containing time series of observed values (typically annual streamflow and rainfall).
#' @param ... Arguments passed to `rstan::sampling` (e.g. iter, chains, cores).
#'
#' `Sim` requires column names of `year` and `Q`.
#' `year` is used to match the simulated an observed values, and
#' `Q` are the simulated values to test, typically streamflow volume/depth
#'#'
#' `Obs` requires column names of `year`, `P` and `Q`.
#' `year` is the year of observation, matched to `year` in `Sim` to calculate errors
#' `Q` is the observed values to compare to the `Q` column from `Sim`, using a log ratio
#' `P` is the independent climate variable, to test if there is a relationship in the errors
#'
#' An optional `ID` column can be included to indicate multiple sites to be used
#' as random factors in the test. If an `ID` column is used, it is required in both `Sim` and `Obs`.
#'
#' @return a list with members:
#' \describe{
#' \item{fit}{rstan model  object for testing of convergence etc}
#' \item{data}{list of data in stan format used to fit the model, after transformation}
#' \item{id_levels}{order of random factors if used}
#' \item{beta_draws}{samples of slope values from the posterior distribution}
#' \item{statistics}{statistics indicating the probability that the slope is: greater than 0 `prob_beta_gt0`, less than 0 `prob_beta_lt0`, and non-zero `p_nonzero`}
#'}
#'
#' @export
#'
#' @examples
#'
#' \donttest{
#' #generate 2 models of annual streamflow
#' samples <- 50
#' #generate some annual rainfall totals
#' Clim <- sort(pmax(0,rnorm(samples,600,100)))
#' #calculate annual runoff from tanh curve
#' observed <- pmax(0,(Clim - 150) - 450 * tanh((Clim - 150)/450))
#' Obs <- data.frame(P=Clim,Q=observed,year=1:length(Clim))
#'
#' #non-robust model, underestimate the wetter years
#' Sim1 <- observed*seq(1,0.7,length.out=length(observed))*rnorm(samples,1,0.05)
#' Sim1 <- data.frame(Q=Sim1,year=1:length(Clim))
#'
#' #robust model, only random error
#' Sim2 <- observed*rnorm(samples,1,0.05)
#' Sim2 <- data.frame(Q=Sim2,year=1:length(Clim))
#'
#' #apply BRRAT test to both, reduce chains and iters to speed up example.
#' fit1 <- BRRAT(Sim1,Obs,chains=1,iter=500)
#' fit2 <- BRRAT(Sim2,Obs,chains=1,iter=500)
#'
#' dat <- data.frame(b = c(fit1$beta_draws,fit2$beta_draws),
#' id = c(rep("fit1",length(fit1$beta_draws)),
#'        rep("fit2",length(fit2$beta_draws))))
#'
#' #fit2 is centered on 0, fit1 has a negative slope
#' ggplot2::ggplot(dat)+
#'  ggplot2::geom_density(ggplot2::aes(b,fill=id))
#'
#' #can also be viewed using the plotting function
#' plot_BRRAT_population(fit1)
#' plot_BRRAT_population(fit2)
#'
#' #Summarise statistics of the mean can be viewed using
#' summarise_BRRAT(fit1)
#' summarise_BRRAT(fit2)
#'}
#'

BRRAT<-function(Sim, Obs, ...){

  #check inputs are OK
  if(!("data.frame" %in% class(Sim))){
    stop("Sim must be of type that inherits data.frame (data.frame, tibble, etc)")
  }

  if(!("data.frame" %in% class(Obs))){
    stop("Obs must be of type that inherits data.frame (data.frame, tibble, etc)")
  }

  expected_cols <- c("year", "Q", "P")
  if(!all(expected_cols %in% names(Obs))){
    stop(paste("Obs input requires columns of:\nyear (index of observations to match to Sim)\nP (independent climate variable)\nQ (observed flow response to evaluate against Sim"))
  }

  if(any(is.na(Sim)) | any(is.na(Obs))) stop("remove NAs")

  if(min(nrow(Sim), nrow(Obs))<3) stop("data frame too small, not enough rows")

  if(min(Obs$Q)<=0) stop("Obs$Q must be positive")

  if(!("ID" %in% names(Obs))){
    warning("No ID column in Obs. Assuming 1 site, adding dummy ID column")
    Obs <- Obs |> dplyr::mutate(ID = "ID")
    Sim <- Sim |> dplyr::mutate(ID = "ID")
  }

  expected_cols <- c("year", "Q", "ID")
  if(!all(expected_cols %in% names(Sim))){
    stop(paste("Sim input requires columns of:\nyear (index of observations to match to Obs)\nQ (Simulated flow response to evaluate against Obs)"))
  }

  #Standardise Rain
  meanP <- Obs |> dplyr::group_by(.data$ID) |> dplyr::summarise(Pm = mean(.data$P),
                                              SD = stats::sd(.data$P))

  Obs <- Obs |> dplyr::left_join(meanP,by="ID") |>
    dplyr::mutate(P=(.data$P-.data$Pm)/.data$SD) |>
    dplyr::select(dplyr::all_of(c("year","P","Obs"="Q","ID")))

  #join data, calculate error as log ratio

  df <- Sim |>
    dplyr::left_join(Obs,by=c("ID","year")) |>
    dplyr::filter(!is.na(.data$Obs)) |> #only years with matching obs
    dplyr::mutate(Qerr = log(.data$Q/.data$Obs),#log ratio
           ID = factor(.data$ID)) |>
    dplyr::select(dplyr::all_of(c("P","Qerr","ID")))

  #setup stan list
  x <- df$P
  y <- df$Qerr
  group <- as.integer(df$ID)
  id_levels <- levels(df$ID)
  J <- length(id_levels)
  N <- nrow(df)

  stan_data <- list(N = N, x = as.vector(x), y = as.vector(y), J = J, group = group)

  if(J==1){
    fit <- rstan::sampling(stanmodels$lm, data = stan_data, ...)
  }else{
    fit <- rstan::sampling(stanmodels$lm_random_id, data = stan_data, ...)
  }


  # Posterior summaries
  summ <- rstan::summary(fit)$summary
  summ_df <- as.data.frame(summ)
  summ_df$param <- rownames(summ)

  # Slope evidence
  beta_draws <- rstan::extract(fit, pars = 'beta')[[1]]
  prob_beta_gt0 <- mean(beta_draws > 0)
  prob_beta_lt0 <- mean(beta_draws < 0)
  p_nonzero <- 2 * min(prob_beta_gt0, prob_beta_lt0)

  out <- list(
    fit = fit, data = stan_data,
    id_levels = id_levels,
    summary = summ_df,
    beta_draws = beta_draws,
    prob_beta_gt0 = prob_beta_gt0, prob_beta_lt0 = prob_beta_lt0, p_nonzero = p_nonzero
  )
  class(out) <- 'BRRAT'

  return(out)
}

test_that("NAs in inputs are rejected", {
  Sim <- data.frame(Q=c(1, 2, NA, 4), year=1:4,ID=1)
  Obs <-data.frame(Q= c(1, 2,  3, 4), P=1:4,year="1:4",ID=1)
  expect_error(BRRAT(Sim, Obs), regexp = "NA", ignore.case = TRUE)
})

test_that("Obs <=0 is rejected",{
	Sim <- data.frame(Q=1:10,year=1:10,ID=1)
	Obs <- data.frame(Q=-1:8,P=1:10,year=1:10,ID=1)
	expect_error(BRRAT(Sim, Obs), regexp = "Obs|positive", ignore.case = TRUE)
})

test_that("Inputs are data frame",{
  Sim <- 1:10
  Obs <- 1:10
  expect_error(BRRAT(Sim, Obs),regexp = "data.frame", ignore.case = TRUE)
})

test_that("data frame has requried headers",{
  Sim <- data.frame(S=1:10,year=1:10,ID=1)
  Obs <- data.frame(A=-1:8,P=1:10,year=1:10,ID=1)
  expect_error(BRRAT(Sim, Obs), regexp = "columns", ignore.case = TRUE)
})

set.seed(10)
samples <- 50
#' #generate some annual rainfall totals
Clim <- sort(pmax(0,rnorm(samples,600,100)))
#calculate annual runoff from tanh curve
observed <- pmax(0,(Clim - 150) - 450 * tanh((Clim - 150)/450))
Obs <- data.frame(P=Clim,Q=observed,year=1:length(Clim),ID=1)
#'
#' #non-robust model, underestimate the wetter years
Sim <- observed*seq(1,0.7,length.out=length(observed))*rnorm(samples,1,0.05)
Sim <- data.frame(Q=Sim,year=1:length(Clim),ID=1)

test_that("BRRAT returns expected object", {
  expect_warning(
  res <- BRRAT(Sim, Obs, iter = iters, warmup = warmup, chains = chains),
  regexp = "Samples Size")

  # Class and core members
  expect_s3_class(res, "BRRAT")
  expect_true(!is.null(res$fit), info = "Stan fit object should be present")

  # Posterior slope draws should be accessible either via $beta or rstan extract
  b <- NULL
  if (!is.null(res$beta)) {
    b <- res$beta
    expect_true(is.numeric(b))
  } else {
    # fall back to extracting from Stan fit
    pars <- try(rstan::extract(res$fit, pars = "beta"), silent = TRUE)
    expect_false(inherits(pars, "try-error"))
    b <- pars$beta
    expect_true(is.numeric(b))
  }

  expect_true(length(b) > 50, info = "Should have many posterior draws")
  expect_true(all(is.finite(b)))
})

set.seed(123)
options(mc.cores = 1L)

# Let tests override Stan sampling args if your BRRAT() forwards "..." to rstan::sampling()
options(BRRAT.iter   = 400L)
options(BRRAT.warmup = 200L)
options(BRRAT.chains = 1L)

iters   <- getOption("BRRAT.iter",   400L)
warmup  <- getOption("BRRAT.warmup", 200L)
chains  <- getOption("BRRAT.chains", 1L)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_helpers.R
\name{export_BRRAT_csv}
\alias{export_BRRAT_csv}
\title{Export global and per-site summaries to CSV}
\usage{
export_BRRAT_csv(fit_obj, file_global, file_per_id, level = 0.95)
}
\arguments{
\item{fit_obj}{A fitted object returned by \code{\link[=BRRAT]{BRRAT()}}, of class \code{BRRAT}.}

\item{file_global}{Path/filename for the global table CSV. No default is
provided; the user must supply an explicit path (e.g. via \code{\link[=tempfile]{tempfile()}} or
a user-chosen location).}

\item{file_per_id}{If applicable to the model, path/filename for the
per-site table CSV. No default is provided; the user must supply an
explicit path.}

\item{level}{Credible interval level passed to \code{\link[=summarise_BRRAT]{summarise_BRRAT()}} (default \code{0.95}).}
}
\value{
(Invisibly) a list with the two data frames written:
\verb{list(global = <data.frame>, per_id = <data.frame>)}.
}
\description{
Writes the output of \code{\link[=summarise_BRRAT]{summarise_BRRAT()}} to two CSV files: one for the
population slope (\code{file_global}) and one for per-site coefficients (\code{file_per_id}).
}
\details{
Export summary tables to CSV files
}
\examples{
\donttest{
set.seed(1)
samples <- 30
Clim <- sort(pmax(0, rnorm(samples, 600, 100)))
observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150) / 450))
Obs <- data.frame(P = Clim, Q = observed, year = 1:samples)
Sim <- data.frame(Q = observed * rnorm(samples, 1, 0.05), year = 1:samples)
res <- BRRAT(Sim, Obs, chains = 1, iter = 500)
export_BRRAT_csv(res,
  file_global = tempfile(fileext = ".csv"),
  file_per_id = tempfile(fileext = ".csv"),
  level = 0.95
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_helpers.R
\name{plot_BRRAT_by_id}
\alias{plot_BRRAT_by_id}
\title{Per-site partial-pooling lines and credible bands}
\usage{
plot_BRRAT_by_id(fit_obj, n_grid = 150, level = 0.8, facet = TRUE)
}
\arguments{
\item{fit_obj}{A fitted object returned by \code{\link[=BRRAT]{BRRAT()}}, of class \code{BRRAT}.}

\item{n_grid}{Integer; number of x points per site for the prediction grids
(default \code{150}).}

\item{level}{Credible interval level (default \code{0.80}) used for site-specific bands.}

\item{facet}{Logical; if \code{TRUE}, facet the plot by \code{ID}; otherwise overlay all sites
with colour and fill aesthetics (default \code{TRUE}).}
}
\value{
A \code{ggplot2} object showing per-site lines and credible intervals.
}
\description{
Plots \strong{site-specific} regression lines using draws of \code{intercept_id[j]} and
\code{slope_id[j]} for each site \code{ID}. Requires more than 1 ID in the initial
regression by \code{\link[=BRRAT]{BRRAT()}}. The output overlays lines and credible bands
across sites; optionally facetted into separate panels for readability.
}
\details{
Plot per-site lines (random intercepts & slopes) with credible intervals

For site \eqn{j}, the line is computed as:
\deqn{\mu_j(x) = \mathrm{intercept\_id}_j + \mathrm{slope\_id}_j \, x_{\mathrm{std}}}{
mu_j(x) = intercept_id[j] + slope_id[j] * x_std}
evaluated on a grid of x values transformed to the centered/scaled space used
during fitting. The function reconstructs equal-tail credible bands by propagating
posterior draws of \code{intercept_id[j]} and \code{slope_id[j]}.
}
\examples{
\donttest{
set.seed(1)
samples <- 30
make_site <- function(id) {
  Clim <- sort(pmax(0, rnorm(samples, 600, 100)))
  observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150) / 450))
  list(
    Obs = data.frame(P = Clim, Q = observed, year = 1:samples, ID = id),
    Sim = data.frame(Q = observed * rnorm(samples, 1, 0.05), year = 1:samples, ID = id)
  )
}
s1 <- make_site("A")
s2 <- make_site("B")
Obs <- rbind(s1$Obs, s2$Obs)
Sim <- rbind(s1$Sim, s2$Sim)
res <- BRRAT(Sim, Obs, chains = 1, iter = 500)
p_sites <- plot_BRRAT_by_id(res, n_grid = 150, level = 0.80, facet = TRUE)
print(p_sites)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BRRAT-package.R
\docType{package}
\name{BRRAT-package}
\alias{BRRAT-package}
\title{The 'BRRAT' package.}
\description{
Function to apply the Bayesian Regression Robustness Assessment Test,
to evaluate if model simulations are independent of the climate input used,
suggesting a robust model.

This package makes use of Stan to fit the linear regression model,
and the precompiled model created for the package with rstantools.
}
\references{
Stan Development Team (NA). RStan: the R interface to Stan. R package version 2.32.6. https://mc-stan.org

Gibbs et al. (2026) in review
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/csiro/hydro_BRRAT_Package}
  \item Report bugs at \url{https://github.com/csiro/hydro_BRRAT_Package/issues}
}

}
\author{
\strong{Maintainer}: Matt Gibbs \email{matt.gibbs@csiro.au}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting_helpers.R
\name{plot_BRRAT_population}
\alias{plot_BRRAT_population}
\title{Population-level line and credible band}
\usage{
plot_BRRAT_population(fit_obj, n_grid = 200, level = 0.95)
}
\arguments{
\item{fit_obj}{A fitted object returned by \code{\link[=BRRAT]{BRRAT()}}, of class \code{BRRAT}.}

\item{n_grid}{Integer; number of x points for the prediction grid (default \code{200}).}

\item{level}{Credible interval level for the band (default \code{0.95}).}
}
\value{
A \code{ggplot2} object showing the population regression line and credible band.
}
\description{
Plots the \strong{population-level} regression line \eqn{\alpha + \beta x} and its
equal-tail credible band by transforming the x-grid to the centered/scaled space
used during fitting. The plot overlays a (potentially downsampled) scatter of the
original observations for context.
}
\details{
Plot population-level regression Qe ~ P with uncertainty band

The function extracts posterior draws of \code{alpha} and \code{beta} from the Stan fit and
evaluates \eqn{\mu(x) = \alpha + \beta x} along a grid of x values on the original
scale, internally converting to the centered/scaled space via
\eqn{x_{\mathrm{std}} = (x - x_{\mathrm{center}}) / x_{\mathrm{scale}}}.
}
\examples{
\donttest{
set.seed(1)
samples <- 30
Clim <- sort(pmax(0, rnorm(samples, 600, 100)))
observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150) / 450))
Obs <- data.frame(P = Clim, Q = observed, year = 1:samples)
Sim <- data.frame(Q = observed * rnorm(samples, 1, 0.05), year = 1:samples)
res <- BRRAT(Sim, Obs, chains = 1, iter = 500)
p_pop <- plot_BRRAT_population(res, n_grid = 200, level = 0.95)
print(p_pop)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BRRAT.R
\name{BRRAT}
\alias{BRRAT}
\title{Bayesian Regression Robustness Assessment Test}
\usage{
BRRAT(Sim, Obs, ...)
}
\arguments{
\item{Sim}{Data frame type containing time series of simulated values (typically annual streamflow).}

\item{Obs}{Data frame type containing time series of observed values (typically annual streamflow and rainfall).}

\item{...}{Arguments passed to \code{rstan::sampling} (e.g. iter, chains, cores).

\code{Sim} requires column names of \code{year} and \code{Q}.
\code{year} is used to match the simulated an observed values, and
\code{Q} are the simulated values to test, typically streamflow volume/depth
#'
\code{Obs} requires column names of \code{year}, \code{P} and \code{Q}.
\code{year} is the year of observation, matched to \code{year} in \code{Sim} to calculate errors
\code{Q} is the observed values to compare to the \code{Q} column from \code{Sim}, using a log ratio
\code{P} is the independent climate variable, to test if there is a relationship in the errors

An optional \code{ID} column can be included to indicate multiple sites to be used
as random factors in the test. If an \code{ID} column is used, it is required in both \code{Sim} and \code{Obs}.}
}
\value{
a list with members:
\describe{
\item{fit}{rstan model  object for testing of convergence etc}
\item{data}{list of data in stan format used to fit the model, after transformation}
\item{id_levels}{order of random factors if used}
\item{beta_draws}{samples of slope values from the posterior distribution}
\item{statistics}{statistics indicating the probability that the slope is: greater than 0 \code{prob_beta_gt0}, less than 0 \code{prob_beta_lt0}, and non-zero \code{p_nonzero}}
}
}
\description{
Function to apply the BRRAT. A linear regression is fit between the error between the transformed \code{Sim}
and \code{Obs} data and the corresponding \code{Clim} data, to test for a dependent
relationship between the error and climate, indicating a model that is not
robust to the input climate data. Stan is used to fit the linear regression,
and hence a distribution of slope values are returned.
}
\details{
Multiple catchments are supported through random factors in the hierarchical
linear regression model,indicated by different values in an \code{ID} column.

Multiple replicates of the Simulated time series are supported, matched based
on the \code{year} column.
}
\examples{

\donttest{
#generate 2 models of annual streamflow
samples <- 50
#generate some annual rainfall totals
Clim <- sort(pmax(0,rnorm(samples,600,100)))
#calculate annual runoff from tanh curve
observed <- pmax(0,(Clim - 150) - 450 * tanh((Clim - 150)/450))
Obs <- data.frame(P=Clim,Q=observed,year=1:length(Clim))

#non-robust model, underestimate the wetter years
Sim1 <- observed*seq(1,0.7,length.out=length(observed))*rnorm(samples,1,0.05)
Sim1 <- data.frame(Q=Sim1,year=1:length(Clim))

#robust model, only random error
Sim2 <- observed*rnorm(samples,1,0.05)
Sim2 <- data.frame(Q=Sim2,year=1:length(Clim))

#apply BRRAT test to both, reduce chains and iters to speed up example.
fit1 <- BRRAT(Sim1,Obs,chains=1,iter=500)
fit2 <- BRRAT(Sim2,Obs,chains=1,iter=500)

dat <- data.frame(b = c(fit1$beta_draws,fit2$beta_draws),
id = c(rep("fit1",length(fit1$beta_draws)),
       rep("fit2",length(fit2$beta_draws))))

#fit2 is centered on 0, fit1 has a negative slope
ggplot2::ggplot(dat)+
 ggplot2::geom_density(ggplot2::aes(b,fill=id))

#can also be viewed using the plotting function
plot_BRRAT_population(fit1)
plot_BRRAT_population(fit2)

#Summarise statistics of the mean can be viewed using
summarise_BRRAT(fit1)
summarise_BRRAT(fit2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_helpers.R
\name{summarise_BRRAT}
\alias{summarise_BRRAT}
\title{Summarise population and site-level posterior summaries}
\usage{
summarise_BRRAT(fit_obj, level = 0.95)
}
\arguments{
\item{fit_obj}{A fitted object returned by \code{\link[=BRRAT]{BRRAT()}}, of class \code{BRRAT}.}

\item{level}{Credible interval level (default \code{0.95}). The lower and upper
intervals are computed as \code{(1 - level) / 2} and \code{1 - (1 - level) / 2}.}
}
\value{
A list with one or two data frames:
\itemize{
\item \code{global}: one row summarising the population slope \code{beta} (mean, sd,
lower/upper credible interval, \code{prob_gt0}, \code{prob_lt0}, and \code{p_nonzero}).
\item \code{per_id}: if applicable (more than one site \code{ID} in the data) one
row per site \code{ID} with posterior summaries for
\code{intercept_id} and \code{slope_id} (mean, sd, lower/upper credible intervals).
}
}
\description{
Computes tidy posterior summaries for the \strong{population slope} (\code{beta}) and
\strong{per-site intercepts and slopes} (from \code{intercept_id} and \code{slope_id} generated
quantities) returned by the Stan fit. The function reports posterior means,
standard deviations, equal-tail credible intervals at a chosen level, and a
two-sided tail probability for non-zero slope at the population level.
}
\details{
Summarise global slope and per-site coefficients into tidy tables

The two-sided tail probability is computed as:
\deqn{p_{\mathrm{nonzero}} = 2\min\{ \Pr(\beta > 0 \mid \mathrm{data}), \Pr(\beta < 0 \mid \mathrm{data}) \}}{
p_nonzero = 2 * min(Pr(beta > 0 | data), Pr(beta < 0 | data))}
which is analogous to a two-sided p-value in spirit, but is a \strong{Bayesian posterior tail
probability}. It shrinks toward 0 as the posterior mass concentrates away from zero.
}
\section{Notes}{

Assumes the Stan model emits \code{intercept_id} and \code{slope_id} in \verb{generated quantities},
i.e., per-site fully-composed coefficients:
\deqn{\mathrm{intercept\_id}_j = \alpha + b_{0j},\quad \mathrm{slope\_id}_j = \beta + b_{1j}}{
intercept_id[j] = alpha + b0[j]; slope_id[j] = beta + b1[j]}
}

\examples{
\donttest{
set.seed(1)
samples <- 30
Clim <- sort(pmax(0, rnorm(samples, 600, 100)))
observed <- pmax(0, (Clim - 150) - 450 * tanh((Clim - 150) / 450))
Obs <- data.frame(P = Clim, Q = observed, year = 1:samples)
Sim <- data.frame(Q = observed * rnorm(samples, 1, 0.05), year = 1:samples)
res <- BRRAT(Sim, Obs, chains = 1, iter = 500)
tabs <- summarise_BRRAT(res, level = 0.95)
tabs$global
head(tabs$per_id)
}

}
